<?php
/**
 * Copyright 2018 Kacper Donat
 *
 * @author Kacper "Kadet" Donat <kacper@kadet.net>
 *
 * Full license available in separate LICENSE file
 */

namespace Kadet\Functional;

use Kadet\Functional\Utils\CurriedFunction;
use Kadet\Functional\Utils\FixedFunction;
use Kadet\Functional\Utils\FunctionPipeline;
use Kadet\Functional\Utils\MemoizedFunction;
use Kadet\Functional\Utils\PartiallyAppliedFunction;

function partial(callable $callable, ...$args): PartiallyAppliedFunction
{
    return new PartiallyAppliedFunction($callable, ...$args);
}

function apply(callable $callable, ...$args)
{
    try {
        $reflection = reflect($callable);
        $required   = $reflection->getNumberOfRequiredParameters();
        $all        = $reflection->getNumberOfParameters();

        switch (true) {
            case $reflection->isVariadic():
                return partial($callable, ...$args);

            case count($args) >= $all:
                $applied = $callable(...array_slice($args, 0, $all));
                return is_callable($applied) ? apply($applied, ...array_slice($args, $all)) : $applied;

            case count($args) >= $required:
                return $callable(...$args);

            default:
                return partial($callable, ...$args);
        }
    } catch (\ReflectionException $exception) {
        // if cannot gather necessary information for intelligent application just use normal one
        return $callable(...$args);
    }
}

function curry(callable $callable, int $threshold = null, ...$args): CurriedFunction
{
    return new CurriedFunction($callable, $threshold, ...$args);
}

function uncurry(CurriedFunction $curried): PartiallyAppliedFunction
{
    return $curried->uncurry();
}

function pipe(...$functions)
{
    return new FunctionPipeline(...$functions);
}

function fix(callable $callable, ...$args)
{
    $f = new FixedFunction($callable);
    return $args ? apply($f, ...$args) : $f;
}

function memoize(callable $callable)
{
    return new MemoizedFunction($callable);
}